==> rest-sdk-0.1.0-sources.jar: com/mulesoft/connectivity/rest/sdk/internal/templating/sdk/SdkTrigger.java <==
/*
 * (c) 2003-2020 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.sdk.internal.templating.sdk;

import static javax.lang.model.element.Modifier.PRIVATE;
import static javax.lang.model.element.Modifier.PROTECTED;
import static javax.lang.model.element.Modifier.PUBLIC;
import static javax.ws.rs.core.MediaType.APPLICATION_JSON_TYPE;
import static javax.ws.rs.core.MediaType.APPLICATION_XML_TYPE;
import static javax.ws.rs.core.MediaType.TEXT_PLAIN_TYPE;
import static org.apache.commons.lang3.StringUtils.isNotBlank;
import static com.mulesoft.connectivity.rest.sdk.internal.util.JavaUtils.getJavaLowerCamelNameFromXml;
import static com.mulesoft.connectivity.rest.sdk.internal.util.JavaUtils.getJavaUpperCamelNameFromXml;

import com.mulesoft.connectivity.rest.commons.internal.util.RestSdkUtils;
import com.mulesoft.connectivity.rest.sdk.api.RestSdkRunConfiguration;
import com.mulesoft.connectivity.rest.sdk.exception.TemplatingException;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorModel;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.trigger.ParameterBinding;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.trigger.Trigger;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.trigger.TriggerParameter;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.type.TypeDefinition;
import com.mulesoft.connectivity.rest.sdk.internal.templating.JavaTemplateEntity;
import com.mulesoft.connectivity.rest.commons.api.source.RequestParameterBinding;
import com.mulesoft.connectivity.rest.commons.api.source.RestPollingSource;
import com.mulesoft.connectivity.rest.commons.internal.util.RestRequestBuilder;

import org.mule.runtime.api.metadata.DataType;
import org.mule.runtime.api.metadata.TypedValue;
import org.mule.runtime.api.util.MultiMap;
import org.mule.runtime.extension.api.annotation.param.Optional;
import org.mule.runtime.extension.api.annotation.param.Parameter;
import org.mule.runtime.extension.api.annotation.param.display.DisplayName;
import org.mule.runtime.extension.api.annotation.param.display.Summary;
import org.mule.runtime.http.api.HttpConstants;

import com.squareup.javapoet.AnnotationSpec;
import com.squareup.javapoet.ClassName;
import com.squareup.javapoet.FieldSpec;
import com.squareup.javapoet.JavaFile;
import com.squareup.javapoet.MethodSpec;
import com.squareup.javapoet.ParameterizedTypeName;
import com.squareup.javapoet.TypeSpec;
import com.squareup.javapoet.WildcardTypeName;

import java.nio.file.Path;

public class SdkTrigger extends JavaTemplateEntity {

  private static final String PATH_TEMPLATE_FIELD = "PATH_TEMPLATE";
  private static final String ITEMS_EXPRESSION_FIELD = "ITEMS_EXPRESSION";
  private static final String WATERMARK_EXPRESSION_FIELD = "WATERMARK_EXPRESSION";
  private static final String IDENTITY_EXPRESSION_FIELD = "IDENTITY_EXPRESSION";
  private static final String REQUEST_BODY_EXPRESSION_FIELD = "REQUEST_BODY_EXPRESSION";

  private static final String WATERMARK_DATA_TYPE_METHOD = "getWatermarkDataType";
  private static final String GET_PATH_TEMPLATE_METHOD = "getPathTemplate";
  private static final String GET_REQUEST_BUILDER_METHOD = "getRequestBuilder";
  private static final String GET_PARAMETER_BINDING_METHOD = "getParameterBinding";
  private static final String GET_PARAMETER_VALUES_METHOD = "getParameterValues";
  private static final String GET_REQUEST_BODY_DATA_TYPE_METHOD = "getRequestBodyDataType";

  private static final String PARAMETER_BINDING_LOCAL_VARIABLE = "parameterBinding";
  private static final String ADD_URI_BINDING_METHOD = "addUriParamBinding";
  private static final String ADD_QUERY_BINDING_METHOD = "addQueryParamBinding";
  private static final String ADD_HEADER_BINDING_METHOD = "addHeaderBinding";

  private static final String PARAMETER_VALUES_LOCAL_VARIABLE = "parameterValues";

  private static final String GET_TYPED_VALUE_OR_NULL_METHOD = "getTypedValueOrNull";

  private static final String XML_STRING_DATA_TYPE = "XML_STRING";
  private static final String JSON_STRING_DATA_TYPE = "JSON_STRING";
  private static final String TEXT_STRING_DATA_TYPE = "TEXT_STRING";

  private final Trigger trigger;

  public SdkTrigger(Path outputDir, ConnectorModel connectorModel, Trigger trigger, RestSdkRunConfiguration runConfiguration) {
    super(outputDir, connectorModel, runConfiguration);
    this.trigger = trigger;
  }

  public String getPackage() {
    return connectorModel.getBasePackage() + ".internal.source";
  }

  public String getJavaClassName() {
    return getJavaUpperCamelNameFromXml(trigger.getInternalName()) + "Trigger";
  }

  private void generateTriggerClass() throws TemplatingException {
    TypeSpec.Builder triggerClassBuilder =
        TypeSpec
            .classBuilder(getJavaClassName())
            .addModifiers(PUBLIC)
            .superclass(RestPollingSource.class)
            .addMethod(generateConstructor())
            .addMethod(generateWatermarkDataTypeMethod())
            .addMethod(generateGetPathTemplateMethod())
            .addMethod(generateGetRequestBuilderMethod())
            .addMethod(generateGetParameterBindingMethod())
            .addMethod(generateGetParameterValuesMethod())
            .addMethod(generateGetRequestBodyDataTypeMethod());

    addClassConstants(triggerClassBuilder);
    addParameters(triggerClassBuilder);

    JavaFile.Builder javaFileBuilder = getJavaFileBuilderForClass(triggerClassBuilder.build(), getPackage());
    javaFileBuilder.addStaticImport(RestSdkUtils.class, GET_TYPED_VALUE_OR_NULL_METHOD);
    writeJavaFile(javaFileBuilder.build());
  }

  private MethodSpec generateGetRequestBodyDataTypeMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_REQUEST_BODY_DATA_TYPE_METHOD)
            .addModifiers(PROTECTED)
            .returns(DataType.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("return $T.$L", DataType.class, getRequestBodyDataType());

    return methodBuilder.build();
  }

  private String getRequestBodyDataType() {
    TypeDefinition inputMetadata = trigger.getOperation().getInputMetadata();

    if (inputMetadata == null || inputMetadata.getMediaType() == null) {
      return JSON_STRING_DATA_TYPE;
    }

    if (inputMetadata.getMediaType().isCompatible(APPLICATION_JSON_TYPE)) {
      return JSON_STRING_DATA_TYPE;
    } else if (inputMetadata.getMediaType().isCompatible(APPLICATION_XML_TYPE)) {
      return XML_STRING_DATA_TYPE;
    } else if (inputMetadata.getMediaType().isCompatible(TEXT_PLAIN_TYPE)) {
      return TEXT_STRING_DATA_TYPE;
    }

    return JSON_STRING_DATA_TYPE;
  }

  private void addParameters(TypeSpec.Builder triggerClassBuilder) {
    for (TriggerParameter parameter : trigger.getParameters()) {
      triggerClassBuilder.addField(getParameter(parameter));
    }
  }

  private FieldSpec getParameter(TriggerParameter parameter) {
    FieldSpec.Builder fieldBuilder =
        FieldSpec.builder(getJavaType(parameter.getParameterType()),
                          getJavaLowerCamelNameFromXml(parameter.getInternalName()),
                          PRIVATE)
            .addAnnotation(Parameter.class)
            .addAnnotation(getAnnotationWithValue(DisplayName.class, parameter.getDisplayName()));

    if (isNotBlank(parameter.getDescription())) {
      fieldBuilder.addAnnotation(getAnnotationWithValue(Summary.class, parameter.getDescription()));
    }

    if (!parameter.isRequired()) {
      fieldBuilder.addAnnotation(Optional.class);
    }

    return fieldBuilder.build();
  }

  private AnnotationSpec getAnnotationWithValue(Class<?> annotationClass, String value) {
    return AnnotationSpec.builder(annotationClass)
        .addMember("value", "$S", value)
        .build();
  }

  private void addClassConstants(TypeSpec.Builder triggerClassBuilder) {
    triggerClassBuilder
        .addField(getConstantStringField(PATH_TEMPLATE_FIELD, trigger.getOperation().getPath()))
        .addField(getConstantStringField(ITEMS_EXPRESSION_FIELD, trigger.getItemsExpression()))
        .addField(getConstantStringField(WATERMARK_EXPRESSION_FIELD, trigger.getWatermarkExpression()))
        .addField(getConstantStringField(IDENTITY_EXPRESSION_FIELD, trigger.getIdentityExpression()))
        .addField(getConstantStringField(REQUEST_BODY_EXPRESSION_FIELD, trigger.getRequestBodyExpression()));
  }

  private MethodSpec generateConstructor() {
    MethodSpec.Builder constructorBuilder =
        MethodSpec.constructorBuilder()
            .addModifiers(PUBLIC);

    constructorBuilder.addStatement("super($L, $L, $L, $L)",
                                    ITEMS_EXPRESSION_FIELD,
                                    WATERMARK_EXPRESSION_FIELD,
                                    IDENTITY_EXPRESSION_FIELD,
                                    REQUEST_BODY_EXPRESSION_FIELD);

    return constructorBuilder.build();
  }

  private MethodSpec generateWatermarkDataTypeMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(WATERMARK_DATA_TYPE_METHOD)
            .addModifiers(PROTECTED)
            .returns(DataType.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("return $T.fromType($T.class)", DataType.class, getJavaType(trigger.getWatermarkType()));

    return methodBuilder.build();
  }

  private MethodSpec generateGetPathTemplateMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_PATH_TEMPLATE_METHOD)
            .addModifiers(PROTECTED)
            .returns(String.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("return $L", PATH_TEMPLATE_FIELD);

    return methodBuilder.build();
  }

  private MethodSpec generateGetRequestBuilderMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_REQUEST_BUILDER_METHOD)
            .addModifiers(PROTECTED)
            .returns(RestRequestBuilder.class)
            .addAnnotation(Override.class)
            .addParameter(String.class, "path");

    methodBuilder.addStatement("return new $T(connection.getBaseUri(), path, $T.$L)",
                               RestRequestBuilder.class,
                               HttpConstants.Method.class,
                               trigger.getOperation().getHttpMethod().toUpperCase());

    return methodBuilder.build();
  }

  private MethodSpec generateGetParameterBindingMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_PARAMETER_BINDING_METHOD)
            .addModifiers(PROTECTED)
            .returns(RequestParameterBinding.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("$1T $2L = new $1T()",
                               RequestParameterBinding.class,
                               PARAMETER_BINDING_LOCAL_VARIABLE);

    if (trigger.getParameterBindings() != null) {
      for (ParameterBinding binding : trigger.getParameterBindings()) {
        methodBuilder.addStatement("$L.$L($S, $S)",
                                   PARAMETER_BINDING_LOCAL_VARIABLE,
                                   getParameterBindingAddMethodName(binding),
                                   binding.getName(),
                                   binding.getExpression());
      }
    }

    methodBuilder.addStatement("return $1L", PARAMETER_BINDING_LOCAL_VARIABLE);

    return methodBuilder.build();
  }

  private String getParameterBindingAddMethodName(ParameterBinding binding) {
    switch (binding.getParameterType()) {
      case QUERY:
        return ADD_QUERY_BINDING_METHOD;
      case HEADER:
        return ADD_HEADER_BINDING_METHOD;
      case URI:
        return ADD_URI_BINDING_METHOD;
    }

    throw new IllegalArgumentException("Parameter type not supported: " + binding.getParameterType());
  }

  private MethodSpec generateGetParameterValuesMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_PARAMETER_VALUES_METHOD)
            .addModifiers(PROTECTED)
            .returns(getParameterValuesMultiMapType())
            .addAnnotation(Override.class);

    methodBuilder.addStatement("final $T $L = new $T<>()",
                               getParameterValuesMultiMapType(),
                               PARAMETER_VALUES_LOCAL_VARIABLE,
                               MultiMap.class);

    for (TriggerParameter parameter : trigger.getParameters()) {
      methodBuilder.addStatement("$L.put($S, $L($L))",
                                 PARAMETER_VALUES_LOCAL_VARIABLE,
                                 parameter.getExternalName(),
                                 GET_TYPED_VALUE_OR_NULL_METHOD,
                                 getJavaLowerCamelNameFromXml(parameter.getInternalName()));
    }

    methodBuilder.addStatement("return $L", PARAMETER_VALUES_LOCAL_VARIABLE);

    return methodBuilder.build();
  }

  private ParameterizedTypeName getParameterValuesMultiMapType() {
    ParameterizedTypeName wildcardTypedValueType =
        ParameterizedTypeName.get(
                                  ClassName.get(TypedValue.class),
                                  WildcardTypeName.subtypeOf(Object.class));

    return ParameterizedTypeName.get(ClassName.get(MultiMap.class), ClassName.get(String.class), wildcardTypedValueType);
  }

  @Override
  public void applyTemplates() throws TemplatingException {
    generateTriggerClass();
  }
}

==> rest-sdk-0.1.1-sources.jar: com/mulesoft/connectivity/rest/sdk/internal/templating/sdk/SdkTrigger.java <==
/*
 * (c) 2003-2020 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.sdk.internal.templating.sdk;

import static com.mulesoft.connectivity.rest.sdk.internal.util.JavaUtils.getJavaLowerCamelNameFromXml;
import static com.mulesoft.connectivity.rest.sdk.internal.util.JavaUtils.getJavaUpperCamelNameFromXml;
import static javax.lang.model.element.Modifier.PRIVATE;
import static javax.lang.model.element.Modifier.PROTECTED;
import static javax.lang.model.element.Modifier.PUBLIC;
import static javax.ws.rs.core.MediaType.APPLICATION_JSON_TYPE;
import static javax.ws.rs.core.MediaType.APPLICATION_XML_TYPE;
import static javax.ws.rs.core.MediaType.TEXT_PLAIN_TYPE;
import static org.apache.commons.lang3.StringUtils.isNotBlank;

import org.mule.runtime.api.metadata.DataType;
import org.mule.runtime.api.metadata.TypedValue;
import org.mule.runtime.api.util.MultiMap;
import org.mule.runtime.extension.api.annotation.metadata.MetadataScope;
import org.mule.runtime.extension.api.annotation.param.Optional;
import org.mule.runtime.extension.api.annotation.param.Parameter;
import org.mule.runtime.extension.api.annotation.param.display.DisplayName;
import org.mule.runtime.extension.api.annotation.param.display.Summary;
import org.mule.runtime.http.api.HttpConstants;
import org.mule.sdk.api.annotation.data.sample.SampleData;

import com.mulesoft.connectivity.rest.commons.api.source.RequestParameterBinding;
import com.mulesoft.connectivity.rest.commons.api.source.RestPollingSource;
import com.mulesoft.connectivity.rest.commons.internal.util.RestRequestBuilder;
import com.mulesoft.connectivity.rest.commons.internal.util.RestSdkUtils;
import com.mulesoft.connectivity.rest.sdk.api.RestSdkRunConfiguration;
import com.mulesoft.connectivity.rest.sdk.exception.TemplatingException;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorModel;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.trigger.ParameterBinding;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.trigger.Trigger;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.trigger.TriggerParameter;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.type.TypeDefinition;
import com.mulesoft.connectivity.rest.sdk.internal.templating.JavaTemplateEntity;

import java.nio.file.Path;

import com.squareup.javapoet.AnnotationSpec;
import com.squareup.javapoet.ClassName;
import com.squareup.javapoet.FieldSpec;
import com.squareup.javapoet.JavaFile;
import com.squareup.javapoet.MethodSpec;
import com.squareup.javapoet.ParameterizedTypeName;
import com.squareup.javapoet.TypeSpec;
import com.squareup.javapoet.WildcardTypeName;

public class SdkTrigger extends JavaTemplateEntity {

  private static final String PATH_TEMPLATE_FIELD = "PATH_TEMPLATE";
  private static final String ITEMS_EXPRESSION_FIELD = "ITEMS_EXPRESSION";
  private static final String WATERMARK_EXPRESSION_FIELD = "WATERMARK_EXPRESSION";
  private static final String IDENTITY_EXPRESSION_FIELD = "IDENTITY_EXPRESSION";
  private static final String REQUEST_BODY_EXPRESSION_FIELD = "REQUEST_BODY_EXPRESSION";

  private static final String WATERMARK_DATA_TYPE_METHOD = "getWatermarkDataType";
  private static final String GET_PATH_TEMPLATE_METHOD = "getPathTemplate";
  private static final String GET_REQUEST_BUILDER_METHOD = "getRequestBuilder";
  private static final String GET_PARAMETER_BINDING_METHOD = "getParameterBinding";
  private static final String GET_PARAMETER_VALUES_METHOD = "getParameterValues";
  private static final String GET_REQUEST_BODY_DATA_TYPE_METHOD = "getRequestBodyDataType";

  private static final String PARAMETER_BINDING_LOCAL_VARIABLE = "parameterBinding";
  private static final String ADD_URI_BINDING_METHOD = "addUriParamBinding";
  private static final String ADD_QUERY_BINDING_METHOD = "addQueryParamBinding";
  private static final String ADD_HEADER_BINDING_METHOD = "addHeaderBinding";

  private static final String PARAMETER_VALUES_LOCAL_VARIABLE = "parameterValues";

  private static final String GET_TYPED_VALUE_OR_NULL_METHOD = "getTypedValueOrNull";

  private static final String XML_STRING_DATA_TYPE = "XML_STRING";
  private static final String JSON_STRING_DATA_TYPE = "JSON_STRING";
  private static final String TEXT_STRING_DATA_TYPE = "TEXT_STRING";

  private final Trigger trigger;
  private final SdkTriggerOutputMetadataResolver outputMetadataResolver;
  private SdkSampleDataProviderTrigger sdkSampleDataProviderTrigger;

  public SdkTrigger(Path outputDir, ConnectorModel connectorModel, SdkConnector sdkConnector, Trigger trigger,
                    RestSdkRunConfiguration runConfiguration)
      throws TemplatingException {
    super(outputDir, connectorModel, runConfiguration);
    this.trigger = trigger;

    this.outputMetadataResolver = buildOutputMetadataResolver(outputDir, connectorModel, sdkConnector, trigger, runConfiguration);
  }

  private SdkTriggerOutputMetadataResolver buildOutputMetadataResolver(Path outputDir, ConnectorModel connectorModel,
                                                                       SdkConnector sdkConnector, Trigger trigger,
                                                                       RestSdkRunConfiguration runConfiguration)
      throws TemplatingException {

    return new SdkTriggerOutputMetadataResolver(outputDir, connectorModel, sdkConnector, trigger.getOperation(),
                                                trigger.getTriggerOutputType(), trigger.getName(), runConfiguration);
  }

  public String getPackage() {
    return connectorModel.getBasePackage() + ".internal.source";
  }

  public String getJavaClassName() {
    return getJavaUpperCamelNameFromXml(trigger.getInternalName()) + "Trigger";
  }

  private void generateTriggerClass() throws TemplatingException {
    TypeSpec.Builder triggerClassBuilder =
        TypeSpec
            .classBuilder(getJavaClassName())
            .addModifiers(PUBLIC)
            .superclass(RestPollingSource.class)
            .addMethod(generateConstructor())
            .addMethod(generateWatermarkDataTypeMethod())
            .addMethod(generateGetPathTemplateMethod())
            .addMethod(generateGetRequestBuilderMethod())
            .addMethod(generateGetParameterBindingMethod())
            .addMethod(generateGetParameterValuesMethod())
            .addMethod(generateGetRequestBodyDataTypeMethod());

    addClassConstants(triggerClassBuilder);
    addParameters(triggerClassBuilder);
    sdkSampleDataProviderTrigger = addSampleData(triggerClassBuilder);

    if (outputMetadataResolver != null && outputMetadataResolver.getRequiresMetadataResolver()) {
      triggerClassBuilder.addAnnotation(getMetadataScopeAnnotation());
    }

    JavaFile.Builder javaFileBuilder = getJavaFileBuilderForClass(triggerClassBuilder.build(), getPackage());
    javaFileBuilder.addStaticImport(RestSdkUtils.class, GET_TYPED_VALUE_OR_NULL_METHOD);
    writeJavaFile(javaFileBuilder.build());
  }

  private AnnotationSpec getMetadataScopeAnnotation() {
    return AnnotationSpec
        .builder(MetadataScope.class)
        .addMember("outputResolver", "$T.class",
                   ClassName.get(outputMetadataResolver.getPackage(),
                                 outputMetadataResolver.getClassName()))
        .build();
  }

  private SdkSampleDataProviderTrigger addSampleData(TypeSpec.Builder triggerClassBuilder) {
    SdkSampleDataProviderTrigger sampleDataProvider = null;
    if (trigger.getSampleData()) {
      sampleDataProvider = new SdkSampleDataProviderTrigger(outputDir, connectorModel, this, trigger, runConfiguration);
      triggerClassBuilder.addAnnotation(generateSampleDataAnnotation(sampleDataProvider));
    }
    return sampleDataProvider;
  }

  private AnnotationSpec generateSampleDataAnnotation(SdkSampleDataProviderTrigger sdkSampleDataProviderTrigger) {
    return AnnotationSpec
        .builder(SampleData.class)
        .addMember(VALUE_MEMBER, "$T.class",
                   ClassName.get(sdkSampleDataProviderTrigger.getPackage(), sdkSampleDataProviderTrigger.getJavaClassName()))
        .build();
  }

  private MethodSpec generateGetRequestBodyDataTypeMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_REQUEST_BODY_DATA_TYPE_METHOD)
            .addModifiers(PROTECTED)
            .returns(DataType.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("return $T.$L", DataType.class, getRequestBodyDataType());

    return methodBuilder.build();
  }

  private String getRequestBodyDataType() {
    TypeDefinition inputMetadata = trigger.getOperation().getInputMetadata();

    if (inputMetadata == null || inputMetadata.getMediaType() == null) {
      return JSON_STRING_DATA_TYPE;
    }

    if (inputMetadata.getMediaType().isCompatible(APPLICATION_JSON_TYPE)) {
      return JSON_STRING_DATA_TYPE;
    } else if (inputMetadata.getMediaType().isCompatible(APPLICATION_XML_TYPE)) {
      return XML_STRING_DATA_TYPE;
    } else if (inputMetadata.getMediaType().isCompatible(TEXT_PLAIN_TYPE)) {
      return TEXT_STRING_DATA_TYPE;
    }

    return JSON_STRING_DATA_TYPE;
  }

  private void addParameters(TypeSpec.Builder triggerClassBuilder) {
    for (TriggerParameter parameter : trigger.getParameters()) {
      triggerClassBuilder.addField(getParameter(parameter));
    }
  }

  private FieldSpec getParameter(TriggerParameter parameter) {
    FieldSpec.Builder fieldBuilder =
        FieldSpec.builder(getJavaType(parameter.getParameterType()),
                          getJavaLowerCamelNameFromXml(parameter.getInternalName()),
                          PRIVATE)
            .addAnnotation(Parameter.class)
            .addAnnotation(getAnnotationWithValue(DisplayName.class, parameter.getDisplayName()));

    if (isNotBlank(parameter.getDescription())) {
      fieldBuilder.addAnnotation(getAnnotationWithValue(Summary.class, parameter.getDescription()));
    }

    if (!parameter.isRequired()) {
      fieldBuilder.addAnnotation(Optional.class);
    }

    return fieldBuilder.build();
  }

  private AnnotationSpec getAnnotationWithValue(Class<?> annotationClass, String value) {
    return AnnotationSpec.builder(annotationClass)
        .addMember("value", "$S", value)
        .build();
  }

  private void addClassConstants(TypeSpec.Builder triggerClassBuilder) {
    triggerClassBuilder
        .addField(getConstantStringField(PATH_TEMPLATE_FIELD, trigger.getOperation().getPath()))
        .addField(getConstantStringField(ITEMS_EXPRESSION_FIELD, trigger.getItemsExpression()))
        .addField(getConstantStringField(WATERMARK_EXPRESSION_FIELD, trigger.getWatermarkExpression()))
        .addField(getConstantStringField(IDENTITY_EXPRESSION_FIELD, trigger.getIdentityExpression()))
        .addField(getConstantStringField(REQUEST_BODY_EXPRESSION_FIELD, trigger.getRequestBodyExpression()));
  }

  private MethodSpec generateConstructor() {
    MethodSpec.Builder constructorBuilder =
        MethodSpec.constructorBuilder()
            .addModifiers(PUBLIC);

    constructorBuilder.addStatement("super($L, $L, $L, $L)",
                                    ITEMS_EXPRESSION_FIELD,
                                    WATERMARK_EXPRESSION_FIELD,
                                    IDENTITY_EXPRESSION_FIELD,
                                    REQUEST_BODY_EXPRESSION_FIELD);

    return constructorBuilder.build();
  }

  private MethodSpec generateWatermarkDataTypeMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(WATERMARK_DATA_TYPE_METHOD)
            .addModifiers(PROTECTED)
            .returns(DataType.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("return $T.fromType($T.class)", DataType.class, getJavaType(trigger.getWatermarkType()));

    return methodBuilder.build();
  }

  private MethodSpec generateGetPathTemplateMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_PATH_TEMPLATE_METHOD)
            .addModifiers(PROTECTED)
            .returns(String.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("return $L", PATH_TEMPLATE_FIELD);

    return methodBuilder.build();
  }

  private MethodSpec generateGetRequestBuilderMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_REQUEST_BUILDER_METHOD)
            .addModifiers(PROTECTED)
            .returns(RestRequestBuilder.class)
            .addAnnotation(Override.class)
            .addParameter(String.class, "path");

    methodBuilder.addStatement("return new $T(connection.getBaseUri(), path, $T.$L)",
                               RestRequestBuilder.class,
                               HttpConstants.Method.class,
                               trigger.getOperation().getHttpMethod().toUpperCase());

    return methodBuilder.build();
  }

  private MethodSpec generateGetParameterBindingMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_PARAMETER_BINDING_METHOD)
            .addModifiers(PROTECTED)
            .returns(RequestParameterBinding.class)
            .addAnnotation(Override.class);

    methodBuilder.addStatement("$1T $2L = new $1T()",
                               RequestParameterBinding.class,
                               PARAMETER_BINDING_LOCAL_VARIABLE);

    if (trigger.getParameterBindings() != null) {
      for (ParameterBinding binding : trigger.getParameterBindings()) {
        methodBuilder.addStatement("$L.$L($S, $S)",
                                   PARAMETER_BINDING_LOCAL_VARIABLE,
                                   getParameterBindingAddMethodName(binding),
                                   binding.getName(),
                                   binding.getExpression());
      }
    }

    methodBuilder.addStatement("return $1L", PARAMETER_BINDING_LOCAL_VARIABLE);

    return methodBuilder.build();
  }

  private String getParameterBindingAddMethodName(ParameterBinding binding) {
    switch (binding.getParameterType()) {
      case QUERY:
        return ADD_QUERY_BINDING_METHOD;
      case HEADER:
        return ADD_HEADER_BINDING_METHOD;
      case URI:
        return ADD_URI_BINDING_METHOD;
    }

    throw new IllegalArgumentException("Parameter type not supported: " + binding.getParameterType());
  }

  private MethodSpec generateGetParameterValuesMethod() {
    MethodSpec.Builder methodBuilder =
        MethodSpec.methodBuilder(GET_PARAMETER_VALUES_METHOD)
            .addModifiers(PROTECTED)
            .returns(getParameterValuesMultiMapType())
            .addAnnotation(Override.class);

    methodBuilder.addStatement("final $T $L = new $T<>()",
                               getParameterValuesMultiMapType(),
                               PARAMETER_VALUES_LOCAL_VARIABLE,
                               MultiMap.class);

    for (TriggerParameter parameter : trigger.getParameters()) {
      methodBuilder.addStatement("$L.put($S, $L($L))",
                                 PARAMETER_VALUES_LOCAL_VARIABLE,
                                 parameter.getExternalName(),
                                 GET_TYPED_VALUE_OR_NULL_METHOD,
                                 getJavaLowerCamelNameFromXml(parameter.getInternalName()));
    }

    methodBuilder.addStatement("return $L", PARAMETER_VALUES_LOCAL_VARIABLE);

    return methodBuilder.build();
  }

  private ParameterizedTypeName getParameterValuesMultiMapType() {
    ParameterizedTypeName wildcardTypedValueType =
        ParameterizedTypeName.get(
                                  ClassName.get(TypedValue.class),
                                  WildcardTypeName.subtypeOf(Object.class));

    return ParameterizedTypeName.get(ClassName.get(MultiMap.class), ClassName.get(String.class), wildcardTypedValueType);
  }

  @Override
  public void applyTemplates() throws TemplatingException {
    if (outputMetadataResolver != null) {
      outputMetadataResolver.applyTemplates();
    }

    generateTriggerClass();

    if (sdkSampleDataProviderTrigger != null) {
      sdkSampleDataProviderTrigger.applyTemplates();
    }
  }
}
